package jossihawk;

import jossihawk.simulator.SimulationProperties;
import jossihawk.simulator.SimulationState;
import jossihawk.simulator.sim.MOEASimulationState;
import jossihawk.simulator.sim.SimulatorController;
import jossihawk.simulator.sim.playback.ScenarioStorer;
import jossihawk.simulator.sim.playback.SimulationPlayback;
import jossihawk.simulator.sim.playback.SimulationRecordingKeeper;

import javax.xml.bind.JAXBException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Scanner;

public class ScenarioCreator {


    public static void main(String[] args) {
        File experimentFolder = new File(SimulationRecordingKeeper.replayFolder);

        File[] filesList = experimentFolder.listFiles();

        System.out.println("Please choose a experiment index");
        for (int i = 0; i < filesList.length; i++) {
            System.out.print(i);
            System.out.println(":\t " + filesList[i].getName());
        }
        Scanner scanner = new Scanner(System.in);
        System.out.print("Enter experiment index: ");

        // get their input as a String
        int index = Integer.parseInt(scanner.next());

        try {
            SimulationPlayback chosenPlayback = SimulationPlayback.readFromFile(new File(filesList[index], "playback.xml"));
            playBack(chosenPlayback);

        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (JAXBException e) {
            e.printStackTrace();
        }
    }

    private static void playBack(SimulationPlayback chosenPlayback) throws JAXBException {
        MOEASimulationState initialState = chosenPlayback.getStates().get(0);
        SimulatorController controller = new SimulatorController(initialState.getState());
        controller.removeAllVehicles();
        controller.setAsMainSimulator(false);

        double lastTime = initialState.getSimulationTime();
        //controller.step(initialState.getSpeedVector());
        while (true) {
            for (int i = 1; i < chosenPlayback.getStates().size(); i++) {
                MOEASimulationState state = chosenPlayback.getStates().get(i);
                controller.removeAllVehicles();
                if (state.getState().getVehicleStates() == null) {
                    continue;
                }
                controller.reinitializeFromState(state.getState());
                controller.step(state.getSpeedVector());
                Scanner scanner = new Scanner(System.in);
                System.out.println("Store? (1 or 0)");
                int store = scanner.nextInt();
                if(store == 1) {
                    SimulationPlayback playback = new SimulationPlayback();
                    playback.addState(state);
                    playback.writeToFile(new File("scenariosForMe"+File.separator+ i+".xml"));
                }
                if (controller.getNumberOfCollisions() > 0) {
                    System.out.println("Crash!");
                }
            }
        }
    }

    private static void storeScenario(SimulationProperties simulationProperties, double simulationTime, SimulationState state, double[] speeds) {
        try {
            ScenarioStorer storer = new ScenarioStorer(simulationProperties.getCarsInScenraio(), simulationProperties.getSpawnRateSouth(), simulationProperties.getSpawnRateNorth(), simulationProperties.getSpawnRateEast(), simulationProperties.getSpawnRateWest());
            SimulationPlayback playback = new SimulationPlayback();
            MOEASimulationState moeaState = new MOEASimulationState(state, speeds, simulationTime);
            playback.addState(moeaState);
            storer.recordSimulationPlayback(playback);
        } catch (IOException e) {
            e.printStackTrace();
        } catch (JAXBException e) {
            e.printStackTrace();
        }
    }
}
