package jossihawk.ep;

import jossihawk.simulator.SimulationProperties;
import jossihawk.utils.ScalingDataPoint;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.Solution;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;

/**
 *
 * Container class for the NondominatedPopulation result in the MOEA Framework.
 * Contains additional methods for picking the individual we prefer.
 *
 */
public class AIMProblemResult implements Iterable<Solution> {

    private final SimulationProperties simulationProperties;
    private NondominatedPopulation moeaResult;
    private final ScalingDataPoint scalingDataPoint;

    public AIMProblemResult(NondominatedPopulation moeaResult, SimulationProperties simulationProperties) {
        this.moeaResult = moeaResult;
        this.simulationProperties = simulationProperties;
        this.scalingDataPoint = null;
    }

    public AIMProblemResult(NondominatedPopulation moeaResult, SimulationProperties simulationProperties, ScalingDataPoint scalingDataPoint) {
        this.moeaResult = moeaResult;
        this.simulationProperties = simulationProperties;
        this.scalingDataPoint = scalingDataPoint;
    }

    public ChosenIndividual getChosenIndividual() {
//        TODO choose the best solution, test the sorting
        ArrayList<Solution> sortedSolutions = new ArrayList<Solution>();
        for (Solution s : moeaResult){
            sortedSolutions.add(s);
        }
        Solution chosenSolution = null;
        if(sortedSolutions.size() > 1) {
            Collections.sort(sortedSolutions, new Comparator<Solution>() {
                public int compare(Solution o1, Solution o2) {
                    if (simulationProperties.getEvaluateCollisions()) {
                        int collisionResult = compareCollision(o1, o2);
                        if (collisionResult != 0) {
                            return collisionResult;
                        }
                        return compareVariables(o1, o2);
                    } else {
                        return compareVariables(o1, o2);
                    }
                }
            });
            chosenSolution = sortedSolutions.get(sortedSolutions.size() - 1);
        } else
            chosenSolution = sortedSolutions.get(0);
        return new ChosenIndividual(chosenSolution,calculateSingleObjectiveValue(chosenSolution));
    }

    private int compareVariables(Solution o1, Solution o2) {
        double o1Score = calculateSingleObjectiveValue(o1);
        double o2Score = calculateSingleObjectiveValue(o2);
        if(o1Score < o2Score)
            return 1;
        else if(o1Score > o2Score)
            return -1;
        else return 0;
    }

    private double calculateSingleObjectiveValue(Solution solution) {
        double solutionScore = 0;
        for (int i = 0; i < solution.getObjectives().length; i++) {
            double objectiveWeight = simulationProperties.getObjectiveWeights().get(i);
            solutionScore += solution.getObjective(i)*objectiveWeight;
        }
        return solutionScore;
    }

    private int compareCollision(Solution o1, Solution o2) {
        double val = o1.getConstraint(0) - o2.getConstraint(0);
        if (val < 0)
            return 1;
        else if(val > 0)
            return -1;
        else return 0;
    }

    public ScalingDataPoint getScalingDataPoint() {
        return scalingDataPoint;
    }

    public Iterator<Solution> iterator() {
        return moeaResult.iterator();
    }
}
