package jossihawk.ep;

import jossihawk.utils.ScalingDataPoint;
import org.moeaframework.core.Algorithm;
import org.moeaframework.core.Solution;
import org.moeaframework.core.TerminationCondition;

public class AIMTerminationCondition implements TerminationCondition {

    private final boolean recordRunStatistics;
    private MOEARunStatistics moeaRunStatistics;
    private ScalingDataPoint scalingDataPoint;
    private int preferredEvaluations;
    private int maxEvaluations;
    private boolean ignoreScaling;

    public AIMTerminationCondition(int preferredEvaluations, int maxEvaluations, double timeMain, boolean isRecordScaling, boolean recordRunStatistics) {
        this.preferredEvaluations = (int) (preferredEvaluations*timeMain);
        this.maxEvaluations = (int) (maxEvaluations*timeMain);
        ignoreScaling = !isRecordScaling;
        scalingDataPoint = null;
        this.recordRunStatistics = recordRunStatistics;
        if(recordRunStatistics) {
            this.moeaRunStatistics = new MOEARunStatistics();
        }
    }

    public void initialize(Algorithm algorithm) {
    }

    public boolean shouldTerminate(Algorithm algorithm) {
        if(recordRunStatistics) {
            moeaRunStatistics.put(algorithm.getNumberOfEvaluations(), algorithm.getResult());
        }

        if(!ignoreScaling) {
            for (Solution solution : algorithm.getResult()) {
                if(!solution.violatesConstraints()) {
                    scalingDataPoint = new ScalingDataPoint(solution.getNumberOfVariables(), algorithm.getNumberOfEvaluations());
                    ignoreScaling = true;
                    break;
                }
            }
        }
        if(algorithm.getNumberOfEvaluations() >= maxEvaluations)
            return true;
        else if (algorithm.getNumberOfEvaluations() >= preferredEvaluations) {
            for (Solution solution : algorithm.getResult()) {
                if (!solution.violatesConstraints())
                    return true;
            }
        }
        return false;
    }

    public ScalingDataPoint getScalingDataPoint() {
        return scalingDataPoint;
    }

    public MOEARunStatistics getMoeaRunStatistics() {
        return moeaRunStatistics;
    }
}
