package jossihawk.ep;

import jossihawk.simulator.AIMSimulator;
import jossihawk.simulator.SimulationProperties;
import jossihawk.simulator.SimulationState;
import org.moeaframework.Executor;
import org.moeaframework.core.NondominatedPopulation;
import org.moeaframework.core.spi.OperatorFactory;

import java.util.Arrays;

public class SingleAIMProblemRunner {

    private AIMProblemResult result;

    private SimulationProperties simulationProperties;

    private Executor executor;

    private AIMTerminationCondition terminationCondition;

    public SingleAIMProblemRunner(SimulationState simulationState) {
        simulationProperties = simulationState.getSimulationProperties();
        OperatorFactory.getInstance().addProvider(new SpeedVectorProvider());
        AIMSimulator simulator = AIMSimulator.getInstance(simulationState);
        simulator.evaluate(new double[simulationState.getSimulationProperties().getNumberOfCars()]);
        double worstStarvation = simulator.getStoppageTimeOfStoppedCars();
        double maxKineticenergyLost = simulator.getKineticEnergyLostInLastTimeStep();
        AIMSimulator simulator1 = AIMSimulator.getInstance(simulationState);
        double[] speeds = new double[simulationState.getSimulationProperties().getNumberOfCars()];
        Arrays.fill(speeds,simulationState.getSimulationProperties().getSpeedLimit());
        simulator1.evaluate(speeds);
        double bestTinyThroughput = simulator1.getCarsEvacuatedFromInnerIntersectionInLastTimeStep();
        double maxDistanceTravelled = simulator1.getTotalDistanceTraveledInLastTimeStep();
        terminationCondition = new AIMTerminationCondition(simulationProperties.getPreferredEvaluations(), simulationProperties.getMaxEvaluations(), simulationProperties.getAlgorithmTimeStepSize(), simulationProperties.isRecordEvaluationsForScaling(), simulationProperties.isRecordMoeaRunStatistics());
        executor = new Executor()
                .withProblemClass(AIMSpeedVectorProblem.class, simulationState, maxKineticenergyLost, maxDistanceTravelled, bestTinyThroughput, worstStarvation)
                .withAlgorithm(simulationProperties.getAlgorithmName())
                .withProperties(simulationProperties.getExecutorSpecificProperties())
                .withTerminationCondition(terminationCondition)
                .distributeOnAllCores();
    }

    public AIMProblemResult run() {
        NondominatedPopulation rawResult = executor.run();
        return result = new AIMProblemResult(rawResult, simulationProperties, terminationCondition.getScalingDataPoint());
    }

    public SimulationProperties getSimulationProperties() {
        return simulationProperties;
    }

    public AIMProblemResult getResult() {
        return result;
    }

    public AIMTerminationCondition getTerminationCondition() {
        return terminationCondition;
    }
}
