package jossihawk.ep;

import org.moeaframework.core.Problem;
import org.moeaframework.core.Solution;
import org.moeaframework.core.Variation;
import org.moeaframework.core.spi.OperatorProvider;

import java.util.Properties;

public class SpeedVectorProvider extends OperatorProvider {

    public final String VARIATION_STRING= "speedVectorVariation";

    @Override
    public String getMutationHint(Problem problem) {
        Solution solution = problem.newSolution();

        for (int i = 0; i < problem.getNumberOfVariables(); i++) {
            if (solution.getVariable(i) instanceof SpeedVectorVariable) {
                return VARIATION_STRING;
            }
        }
        return null;
    }

    @Override
    public String getVariationHint(Problem problem) {
        Solution solution = problem.newSolution();
        for (int i = 0; i < problem.getNumberOfVariables(); i++) {
            if (solution.getVariable(i) instanceof SpeedVectorVariable) {
                return VARIATION_STRING;
            }
        }
        return null;
    }

    @Override
    public Variation getVariation(String name, Properties properties, Problem problem) {
        double mutationRate = Double.parseDouble(properties.getProperty("mutationRate"));
        double speedLimit = Double.parseDouble(properties.getProperty("speedLimit"));
        double crossoverRate = Double.parseDouble(properties.getProperty("crossoverRate"));
        if(name.equalsIgnoreCase(VARIATION_STRING)){
            return new SpeedVectorVariation(mutationRate, speedLimit, crossoverRate);
        }
        return null;
    }
}
