package jossihawk.ep;

import org.moeaframework.core.PRNG;
import org.moeaframework.core.Solution;
import org.moeaframework.core.Variation;

public class SpeedVectorVariation implements Variation {

    private double mutationRate;
    private double speedLimit;
    private double crossoverRate;

    public SpeedVectorVariation(double mutationRate, double speedLimit, double crossoverRate) {
        this.mutationRate = mutationRate;
        this.speedLimit = speedLimit;
        this.crossoverRate = crossoverRate;
    }

    public int getArity() {
        return 2;
    }

    public Solution[] evolve(Solution[] solutions) {
        Solution child1 = solutions[0].deepCopy();
        Solution child2 = solutions[1].deepCopy();
        Solution[] result = doCrossover(new Solution[]{child1, child2});
        child1 = mutate(result[0]);
        child2 = mutate(result[1]);
        return new Solution[]{child1, child2};
    }

    private Solution[] doCrossover(Solution[] solutions) {
        if (PRNG.nextDouble(0, 1) < crossoverRate) {
            int solution_length = solutions[0].getNumberOfVariables();
            int separator = PRNG.nextInt(0, solution_length);
            Solution left = solutions[0];
            Solution right = solutions[1];
            for (int i = 0; i < separator; i++) {
                SpeedVectorVariable tempSpeedVar1 = ((SpeedVectorVariable) left.getVariable(i));
                SpeedVectorVariable tempSpeedVar2 = ((SpeedVectorVariable) right.getVariable(i));
                left.setVariable(i, tempSpeedVar2);
                right.setVariable(i, tempSpeedVar1);
            }
            return new Solution[]{left, right};
        } else return solutions;
    }

    private Solution mutate(Solution solution) {
        int solution_length = solution.getNumberOfVariables();
        for (int i = 0; i < solution_length; i++) {
            SpeedVectorVariable var = (SpeedVectorVariable) solution.getVariable(i);
            if (PRNG.nextDouble(0, 1) < mutationRate) {
                mutateSpeedVector(var);
            }
        }
        return solution;
    }

    private void mutateSpeedVector(SpeedVectorVariable var) {
        double val = PRNG.nextDouble(-2, speedLimit + 2);
        if (val < 0) val = 0.0;
        else if (val > speedLimit) val = speedLimit;
        var.setSpeed(val);
    }

}
