package jossihawk.simulator;

import jossihawk.simulator.sim.SimulatorController;

public abstract class AIMSimulator {

    public static AIMSimulator getInstance(SimulationState simulationState) {
        SimulationProperties simulationProperties = simulationState.getSimulationProperties();
        if(simulationProperties.getSimulatorType().equals("test")) {
            return new TestSimulatorImpl(simulationProperties);
        } else if(simulationProperties.getSimulatorType().equals("prod")) {
            return new SimulatorController(simulationState);
        }
        else {
            throw new IllegalArgumentException("You need to specify a legal AIMSimulator in the config file");
        }
    }

    public abstract int getNumberOfActiveCars();

    public abstract void step(double[] speeds);

    public abstract void evaluate(double[] speeds);

    public abstract AIMStats getTotalStats();

    public abstract int getNumberOfCollisions();
    public abstract int getThroughput();

    public abstract SimulationState getState();
    public abstract void setSpawnVehicles(boolean spawnVehicles);
    public abstract void setAsMainSimulator(boolean headless);
    public abstract double getSimulationTime();

    public abstract double getTotalDistanceTraveledInLastTimeStep();
    public abstract double getKineticEnergyLostInLastTimeStep();
    public abstract double getCarsEvacuatedFromInnerIntersectionInLastTimeStep();
    public abstract double getStoppageTimeOfStoppedCars();
}
