package jossihawk.simulator;

import jossihawk.ep.MOEARunStatistics;
import jossihawk.simulator.sim.stats.VehicleStatistics;
import jossihawk.utils.ScalingDataPoint;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.List;
import java.util.Map;

@XmlRootElement
public class AIMStats {

    private double meanEvacTime;

    private double totalSpentEk;

    private int throughput;

    private int nofChrashedVehicles;
    @XmlElement
    private Map<Integer, VehicleStatistics> vehicleStatisticsMap;

    private List<ScalingDataPoint> scalingDataPoints;

    private List<MOEARunStatistics> moeaRunStatistics;

    public AIMStats() {
    }

    public AIMStats(Map<Integer, VehicleStatistics> vehicleStatisticsMap, int nofChrashedVehicles) {
        this.vehicleStatisticsMap = vehicleStatisticsMap;
        double totalEvacTime = 0;
        totalSpentEk = 0;
        throughput = 0;
        this.nofChrashedVehicles = nofChrashedVehicles;
        for (VehicleStatistics vehicleStatistics : vehicleStatisticsMap.values()) {
            if (vehicleStatistics.getEvacuationTime() != 0.0) {
                totalEvacTime += vehicleStatistics.getEvacuationTime() - vehicleStatistics.getSpawnTime();
                throughput++;
            }
            totalSpentEk += vehicleStatistics.getSpentEk();
        }
        if (throughput > 0)
            meanEvacTime = totalEvacTime / throughput;
        else meanEvacTime = 0;
    }

    public AIMStats(Map<Integer, VehicleStatistics> vehicleStatisticsMap, int nofChrashedVehicles, List<ScalingDataPoint> scalingDataPoints, List<MOEARunStatistics> moeaRunStatistics) {
        this.vehicleStatisticsMap = vehicleStatisticsMap;
        this.moeaRunStatistics = moeaRunStatistics;
        this.scalingDataPoints = scalingDataPoints;
        double totalEvacTime = 0;
        totalSpentEk = 0;
        throughput = 0;
        this.nofChrashedVehicles = nofChrashedVehicles;
        for (VehicleStatistics vehicleStatistics : vehicleStatisticsMap.values()) {
            if (vehicleStatistics.getEvacuationTime() != 0.0) {
                totalEvacTime += vehicleStatistics.getEvacuationTime() - vehicleStatistics.getSpawnTime();
                throughput++;
            }
            totalSpentEk += vehicleStatistics.getSpentEk();
        }
        if (throughput > 0)
            meanEvacTime = totalEvacTime / throughput;
        else meanEvacTime = 0;
    }
    /**
     * Takes a File object, then writes its current state as xml to said file.
     * Useful for storing playbacks for later use.
     */
    public void writeToFile(File file) throws JAXBException {
        JAXBContext context = JAXBContext.newInstance(AIMStats.class);
        Marshaller m = context.createMarshaller();
        m.marshal(this, file);
    }

    /**
     * Reads a stored simulation run from a given xml file.
     *
     * @param file
     * @throws FileNotFoundException
     */
    public static AIMStats readFromFile(File file) throws FileNotFoundException, JAXBException {
        JAXBContext context = JAXBContext.newInstance(AIMStats.class);
        Unmarshaller m = context.createUnmarshaller();
        return (AIMStats) m.unmarshal(file);
    }

    public int getNofChrashedVehicles() {
        return nofChrashedVehicles;
    }

    public double getMeanEvacTime() {
        return meanEvacTime;
    }

    public double getTotalSpentEk() {
        return totalSpentEk;
    }

    public int getThroughput() {
        return throughput;
    }

    public Map<Integer, VehicleStatistics> getVehicleStatisticsMap() {
        return vehicleStatisticsMap;
    }

    public void setMeanEvacTime(double meanEvacTime) {
        this.meanEvacTime = meanEvacTime;
    }

    public void setTotalSpentEk(double totalSpentEk) {
        this.totalSpentEk = totalSpentEk;
    }

    public void setThroughput(int throughput) {
        this.throughput = throughput;
    }

    public void setNofChrashedVehicles(int nofChrashedVehicles) {
        this.nofChrashedVehicles = nofChrashedVehicles;
    }

    @Override
    public String toString() {
        return "AIMStats{" +
                "meanEvacTime=" + meanEvacTime +
                ", totalSpentEk=" + totalSpentEk +
                ", throughput=" + throughput +
                ", nofChrashedVehicles=" + nofChrashedVehicles +
                ", vehicleStatisticsMap=" + vehicleStatisticsMap +
                '}';
    }

    public void setScalingDataPoints(List<ScalingDataPoint> scalingDataPoints) {
        this.scalingDataPoints = scalingDataPoints;
    }

    public List<ScalingDataPoint> getScalingDataPoints() {
        return scalingDataPoints;
    }

    public void setMoeaRunStatistics(List<MOEARunStatistics> moeaRunStatistics) {
        this.moeaRunStatistics = moeaRunStatistics;
    }

    public List<MOEARunStatistics> getMoeaRunStatistics() {
        return moeaRunStatistics;
    }
}


