package jossihawk.simulator;


import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

public class SimulationProperties {

    private boolean spawnVehiclesInMain;
    private int numberOfCars, numberOfObjectives, numberOfSteps, numberOfLanes, preferredEvaluations, maxEvaluations, carsInScenraio;
    private boolean evaluateCollisions, evaluateThroughput, evaluateDistanceTravelled, constrainCollisions,
            recordForPlayback, evaluateEk, evaluateStarvation, evaluateTinyThroughput, evaluateInvertedDistanceTravelled;
    private double speedLimit, evaluationSimulationTime, algorithmTimeStepSize;
    private String algorithmName, simulatorType, title;
    private Properties executorSpecificProperties;
    private List<Double> objectiveWeights;
    private boolean graphChosenIndividual, isImportantRun, showMETHistogram, isScenarioCreation, headless, startScenario, recordEvaluationsForScaling;
    private double spawnRateNorth, spawnRateEast, spawnRateWest, spawnRateSouth;

    public SimulationProperties() {
    }

    public SimulationProperties(String filepath) {
        Properties rawSimulationProperties = readProperties(filepath);

        numberOfCars = Integer.parseInt(rawSimulationProperties.getProperty("numberOfCars"));
        numberOfSteps = Integer.parseInt(rawSimulationProperties.getProperty("numberOfSteps"));
        numberOfLanes = Integer.parseInt(rawSimulationProperties.getProperty("numberOfLanes"));
        preferredEvaluations = Integer.parseInt(rawSimulationProperties.getProperty("preferredEvaluations"));
        maxEvaluations = Integer.parseInt(rawSimulationProperties.getProperty("maxEvaluations"));

        carsInScenraio = Integer.parseInt(rawSimulationProperties.getProperty("carsInScenraio"));

        algorithmName = rawSimulationProperties.getProperty("algorithmName");
        title = rawSimulationProperties.getProperty("title");
        simulatorType = rawSimulationProperties.getProperty("simulatorType");

//        Put the executor properties directly in a Properties object - feed directly to Executor.
        Properties tempExecutorSpecificProperties = new Properties();
        tempExecutorSpecificProperties.setProperty("populationSize", rawSimulationProperties.getProperty("populationSize"));
        tempExecutorSpecificProperties.setProperty("speedLimit", rawSimulationProperties.getProperty("speedLimit"));
        tempExecutorSpecificProperties.setProperty("mutationRate", rawSimulationProperties.getProperty("mutationRate"));
        tempExecutorSpecificProperties.setProperty("crossoverRate", rawSimulationProperties.getProperty("crossoverRate"));
        executorSpecificProperties = tempExecutorSpecificProperties;

        speedLimit = Double.parseDouble(rawSimulationProperties.getProperty("speedLimit"));
        evaluationSimulationTime = Double.parseDouble(rawSimulationProperties.getProperty("evaluationSimulationTime"));
        algorithmTimeStepSize = Double.parseDouble(rawSimulationProperties.getProperty("algorithmTimeStepSize"));

        //Directional spawn rates
        spawnRateNorth = Double.parseDouble(rawSimulationProperties.getProperty("spawnRateNorth"));
        spawnRateEast = Double.parseDouble(rawSimulationProperties.getProperty("spawnRateEast"));
        spawnRateSouth = Double.parseDouble(rawSimulationProperties.getProperty("spawnRateSouth"));
        spawnRateWest = Double.parseDouble(rawSimulationProperties.getProperty("spawnRateWest"));

        // Booleans
        graphChosenIndividual = Boolean.parseBoolean(rawSimulationProperties.getProperty("graphChosenIndividual"));
        showMETHistogram = Boolean.parseBoolean(rawSimulationProperties.getProperty("showMETHistogram"));
        isScenarioCreation = Boolean.parseBoolean(rawSimulationProperties.getProperty("isScenarioCreation"));
        headless = Boolean.parseBoolean(rawSimulationProperties.getProperty("headless"));
        startScenario = Boolean.parseBoolean(rawSimulationProperties.getProperty("startScenario"));
        spawnVehiclesInMain = Boolean.parseBoolean(rawSimulationProperties.getProperty("spawnVehiclesInMain"));
        recordEvaluationsForScaling = Boolean.parseBoolean(rawSimulationProperties.getProperty("recordEvaluationsForScaling"));

//        Set the boolean values for objectives and count how many is true
        constrainCollisions = Boolean.parseBoolean(rawSimulationProperties.getProperty("constrainCollisions"));
        int countObjectives = 0;
        if (evaluateCollisions = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateCollisions")))
            countObjectives++;
        if (evaluateThroughput = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateThroughput")))
            countObjectives++;
        if (evaluateDistanceTravelled = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateDistanceTravelled")))
            countObjectives++;
        if (evaluateEk = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateEk")))
            countObjectives++;
        if (evaluateStarvation = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateStarvation")))
            countObjectives++;
        if (evaluateTinyThroughput = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateTinyThroughput")))
            countObjectives++;
        if (evaluateInvertedDistanceTravelled = Boolean.parseBoolean(rawSimulationProperties.getProperty("evaluateInvertedDistanceTravelled")))
            countObjectives++;
        numberOfObjectives = countObjectives;

        String[] objectiveWeightsStringArray = rawSimulationProperties.get("objectiveWeights").toString().split(",");
        objectiveWeights = new ArrayList<Double>();
        for (int i = 0; i < objectiveWeightsStringArray.length; i++) {
            objectiveWeights.add(Double.parseDouble(objectiveWeightsStringArray[i]));
        }

        //recording values
        recordForPlayback = Boolean.parseBoolean(rawSimulationProperties.getProperty("recordForPlayback"));
        isImportantRun = Boolean.parseBoolean(rawSimulationProperties.getProperty("isImportantRun"));
    }

    public SimulationProperties(SimulationProperties simulationProperties, int numberOfCars) {
        this.numberOfCars = numberOfCars;
        algorithmName = simulationProperties.getAlgorithmName();
        executorSpecificProperties = simulationProperties.getExecutorSpecificProperties();
        speedLimit = simulationProperties.getSpeedLimit();
        evaluateCollisions = simulationProperties.getEvaluateCollisions();
        evaluateThroughput = simulationProperties.getEvaluateThroughput();
        numberOfObjectives = simulationProperties.getNumberOfObjectives();
        numberOfSteps = simulationProperties.getNumberOfSteps();
        simulatorType = simulationProperties.getSimulatorType();
        numberOfLanes = simulationProperties.getNumberOfLanes();
        evaluationSimulationTime = simulationProperties.getEvaluationSimulationTime();
        algorithmTimeStepSize = simulationProperties.getAlgorithmTimeStepSize();
        constrainCollisions = simulationProperties.getConstrainCollisions();
        objectiveWeights = simulationProperties.getObjectiveWeights();
        preferredEvaluations = simulationProperties.getPreferredEvaluations();
        graphChosenIndividual = simulationProperties.getGraphChosenIndividual();
        maxEvaluations = simulationProperties.getMaxEvaluations();
        evaluateDistanceTravelled = simulationProperties.isEvaluateDistanceTravelled();
        evaluateEk = simulationProperties.isEvaluateEk();
        evaluateStarvation = simulationProperties.isEvaluateStarvation();
        evaluateTinyThroughput = simulationProperties.isEvaluateTinyThroughput();
        title = simulationProperties.getTitle();
        isScenarioCreation = simulationProperties.isScenarioCreation();
        carsInScenraio = simulationProperties.getCarsInScenraio();
        startScenario = simulationProperties.isStartScenario();
        headless = simulationProperties.isHeadless();
        spawnVehiclesInMain = simulationProperties.isSpawnVehiclesInMain();
        evaluateInvertedDistanceTravelled = simulationProperties.isEvaluateInvertedDistanceTravelled();
        recordEvaluationsForScaling = simulationProperties.isRecordEvaluationsForScaling();
    }

    public boolean getGraphChosenIndividual() {
        return graphChosenIndividual;
    }


    /**
     * Read the properties file
     *
     * @param filename The path to the properties file
     * @return Properties object containing key/value pairs
     */
    private Properties readProperties(String filename) {
        Properties properties = new Properties();
        InputStream input = null;

        try {
            input = new FileInputStream(filename);
            properties.load(input);

        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            if (input != null) {
                try {
                    input.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return properties;
    }

    public List<String> getObjectiveNames() {
        List<String> objectiveNames = new ArrayList<String>();
        if (getEvaluateCollisions()) {
            objectiveNames.add("Collisions");
        }
        if (isEvaluateDistanceTravelled()) {
            objectiveNames.add("Distance Travelled");
        }
        if (getEvaluateThroughput()) {
            objectiveNames.add("Throughput");
        }
        if (isEvaluateEk()) {
            objectiveNames.add("Ek");
        }
        if (isEvaluateStarvation()) {
            objectiveNames.add("Starvation");
        }
        if (isEvaluateTinyThroughput()) {
            objectiveNames.add("Tiny-throughput");
        }
        if(isEvaluateInvertedDistanceTravelled())
            objectiveNames.add("Inverted Distance Travelled");
        return objectiveNames;
    }

    public boolean isStartScenario() {
        return startScenario;
    }

    public void setStartScenario(boolean startScenario) {
        this.startScenario = startScenario;
    }

    public int getCarsInScenraio() {
        return carsInScenraio;
    }

    public void setCarsInScenraio(int carsInScenraio) {
        this.carsInScenraio = carsInScenraio;
    }

    public boolean isScenarioCreation() {
        return isScenarioCreation;
    }

    public void setScenarioCreation(boolean scenarioCreation) {
        isScenarioCreation = scenarioCreation;
    }

    public boolean isImportantRun() {
        return isImportantRun;
    }

    public double getSpawnRateNorth() {
        return spawnRateNorth;
    }

    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public void setSpawnRateNorth(double spawnRateNorth) {
        this.spawnRateNorth = spawnRateNorth;
    }

    public double getSpawnRateEast() {
        return spawnRateEast;
    }

    public void setSpawnRateEast(double spawnRateEast) {
        this.spawnRateEast = spawnRateEast;
    }

    public double getSpawnRateWest() {
        return spawnRateWest;
    }

    public void setSpawnRateWest(double spawnRateWest) {
        this.spawnRateWest = spawnRateWest;
    }

    public double getSpawnRateSouth() {
        return spawnRateSouth;
    }

    public void setSpawnRateSouth(double spawnRateSouth) {
        this.spawnRateSouth = spawnRateSouth;
    }

    public void setImportantRun(boolean importantRun) {
        isImportantRun = importantRun;
    }

    public boolean isShowMETHistogram() {
        return showMETHistogram;
    }

    public void setShowMETHistogram(boolean showMETHistogram) {
        this.showMETHistogram = showMETHistogram;
    }

    public int getNumberOfCars() {
        return numberOfCars;
    }

    public boolean getEvaluateCollisions() {
        return evaluateCollisions;
    }

    public boolean getEvaluateThroughput() {
        return evaluateThroughput;
    }

    public boolean getConstrainCollisions() {
        return constrainCollisions;
    }

    public int getNumberOfObjectives() {
        return numberOfObjectives;
    }

    public int getNumberOfLanes() {
        return numberOfLanes;
    }

    public double getSpeedLimit() {
        return speedLimit;
    }

    public int getPopulationSize() {
        return Integer.parseInt(executorSpecificProperties.getProperty("populationSize"));
    }

    public void setMaxEvaluations(int maxEvaluations) {
        this.maxEvaluations = maxEvaluations;
    }

    public int getMaxEvaluations() {
        return maxEvaluations;
    }

    public Properties getExecutorSpecificProperties() {
        return executorSpecificProperties;
    }

    public String getAlgorithmName() {
        return algorithmName;
    }

    public SimulationProperties createNewSimulationProperties(int numberOfCars) {
        return new SimulationProperties(this, numberOfCars);
    }

    public int getNumberOfSteps() {
        return numberOfSteps;
    }

    public String getSimulatorType() {
        return simulatorType;
    }

    public double getEvaluationSimulationTime() {
        return evaluationSimulationTime;
    }

    public double getAlgorithmTimeStepSize() {
        return algorithmTimeStepSize;
    }

    public List<Double> getObjectiveWeights() {
        return objectiveWeights;
    }

    public int getPreferredEvaluations() {
        return preferredEvaluations;
    }

    public void setNumberOfCars(int numberOfCars) {
        this.numberOfCars = numberOfCars;
    }

    public void setNumberOfObjectives(int numberOfObjectives) {
        this.numberOfObjectives = numberOfObjectives;
    }

    public void setNumberOfSteps(int numberOfSteps) {
        this.numberOfSteps = numberOfSteps;
    }

    public void setNumberOfLanes(int numberOfLanes) {
        this.numberOfLanes = numberOfLanes;
    }

    public void setEvaluateCollisions(boolean evaluateCollisions) {
        this.evaluateCollisions = evaluateCollisions;
    }

    public void setEvaluateThroughput(boolean evaluateThroughput) {
        this.evaluateThroughput = evaluateThroughput;
    }

    public void setConstrainCollisions(boolean constrainCollisions) {
        this.constrainCollisions = constrainCollisions;
    }

    public void setSpeedLimit(double speedLimit) {
        this.speedLimit = speedLimit;
    }

    public void setEvaluationSimulationTime(double evaluationSimulationTime) {
        this.evaluationSimulationTime = evaluationSimulationTime;
    }

    public void setPreferredEvaluations(int preferredEvaluations) {
        this.preferredEvaluations = preferredEvaluations;
    }

    public void setAlgorithmTimeStepSize(double algorithmTimeStepSize) {
        this.algorithmTimeStepSize = algorithmTimeStepSize;
    }

    public void setAlgorithmName(String algorithmName) {
        this.algorithmName = algorithmName;
    }

    public void setSimulatorType(String simulatorType) {
        this.simulatorType = simulatorType;
    }

    public void setExecutorSpecificProperties(Properties executorSpecificProperties) {
        this.executorSpecificProperties = executorSpecificProperties;
    }

    public void setObjectiveWeights(List<Double> objectiveWeights) {
        this.objectiveWeights = objectiveWeights;
    }

    public void setGraphChosenIndividual(boolean graphChosenIndividual) {
        this.graphChosenIndividual = graphChosenIndividual;
    }

    public boolean isEvaluateDistanceTravelled() {
        return evaluateDistanceTravelled;
    }

    public boolean isEvaluateEk() {
        return evaluateEk;
    }

    public boolean isEvaluateStarvation() {
        return evaluateStarvation;
    }

    public boolean isEvaluateTinyThroughput() {
        return evaluateTinyThroughput;
    }

    public void setEvaluateEk(boolean evaluateEk) {
        this.evaluateEk = evaluateEk;
    }

    public void setEvaluateStarvation(boolean evaluateStarvation) {
        this.evaluateStarvation = evaluateStarvation;
    }

    public void setEvaluateTinyThroughput(boolean evaluateTinyThroughput) {
        this.evaluateTinyThroughput = evaluateTinyThroughput;
    }

    public void setEvaluateDistanceTravelled(boolean evaluateDistanceTravelled) {
        this.evaluateDistanceTravelled = evaluateDistanceTravelled;
    }

    public boolean isRecordForPlayback() {
        return recordForPlayback;
    }

    public void setRecordForPlayback(boolean recordForPlayback) {
        this.recordForPlayback = recordForPlayback;
    }

    public boolean isHeadless() {
        return headless;
    }

    public void setHeadless(boolean headless) {
        this.headless = headless;
    }

    public boolean isSpawnVehiclesInMain() {
        return spawnVehiclesInMain;
    }

    public void setSpawnVehiclesInMain(boolean spawnVehiclesInMain) {
        this.spawnVehiclesInMain = spawnVehiclesInMain;
    }

    public boolean isEvaluateInvertedDistanceTravelled() {
        return evaluateInvertedDistanceTravelled;
    }

    public boolean isRecordEvaluationsForScaling() {
        return true;
    }

    public void setEvaluateInvertedDistanceTravelled(boolean evaluateInvertedDistanceTravelled) {
        this.evaluateInvertedDistanceTravelled = evaluateInvertedDistanceTravelled;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof SimulationProperties)) return false;

        SimulationProperties that = (SimulationProperties) o;

        if (getNumberOfCars() != that.getNumberOfCars()) return false;
        if (getNumberOfObjectives() != that.getNumberOfObjectives()) return false;
        if (getNumberOfSteps() != that.getNumberOfSteps()) return false;
        if (getNumberOfLanes() != that.getNumberOfLanes()) return false;
        if (getPreferredEvaluations() != that.getPreferredEvaluations()) return false;
        if (getMaxEvaluations() != that.getMaxEvaluations()) return false;
        if (getCarsInScenraio() != that.getCarsInScenraio()) return false;
        if (getEvaluateCollisions() != that.getEvaluateCollisions()) return false;
        if (getEvaluateThroughput() != that.getEvaluateThroughput()) return false;
        if (isEvaluateDistanceTravelled() != that.isEvaluateDistanceTravelled()) return false;
        if (getConstrainCollisions() != that.getConstrainCollisions()) return false;
        if (isRecordForPlayback() != that.isRecordForPlayback()) return false;
        if (isEvaluateEk() != that.isEvaluateEk()) return false;
        if (isEvaluateStarvation() != that.isEvaluateStarvation()) return false;
        if (isEvaluateTinyThroughput() != that.isEvaluateTinyThroughput()) return false;
        if (Double.compare(that.getSpeedLimit(), getSpeedLimit()) != 0) return false;
        if (Double.compare(that.getEvaluationSimulationTime(), getEvaluationSimulationTime()) != 0) return false;
        if (Double.compare(that.getAlgorithmTimeStepSize(), getAlgorithmTimeStepSize()) != 0) return false;
        if (getGraphChosenIndividual() != that.getGraphChosenIndividual()) return false;
        if (isImportantRun() != that.isImportantRun()) return false;
        if (isShowMETHistogram() != that.isShowMETHistogram()) return false;
        if (isScenarioCreation() != that.isScenarioCreation()) return false;
        if (isHeadless() != that.isHeadless()) return false;
        if (isStartScenario() != that.isStartScenario()) return false;
        if (Double.compare(that.getSpawnRateNorth(), getSpawnRateNorth()) != 0) return false;
        if (Double.compare(that.getSpawnRateEast(), getSpawnRateEast()) != 0) return false;
        if (Double.compare(that.getSpawnRateWest(), getSpawnRateWest()) != 0) return false;
        if (Double.compare(that.getSpawnRateSouth(), getSpawnRateSouth()) != 0) return false;
        if (getAlgorithmName() != null ? !getAlgorithmName().equals(that.getAlgorithmName()) : that.getAlgorithmName() != null)
            return false;
        if (getSimulatorType() != null ? !getSimulatorType().equals(that.getSimulatorType()) : that.getSimulatorType() != null)
            return false;
        if (getTitle() != null ? !getTitle().equals(that.getTitle()) : that.getTitle() != null) return false;
        if (getExecutorSpecificProperties() != null ? !getExecutorSpecificProperties().equals(that.getExecutorSpecificProperties()) : that.getExecutorSpecificProperties() != null)
            return false;
        return getObjectiveWeights() != null ? getObjectiveWeights().equals(that.getObjectiveWeights()) : that.getObjectiveWeights() == null;

    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        result = getNumberOfCars();
        result = 31 * result + getNumberOfObjectives();
        result = 31 * result + getNumberOfSteps();
        result = 31 * result + getNumberOfLanes();
        result = 31 * result + getPreferredEvaluations();
        result = 31 * result + getMaxEvaluations();
        result = 31 * result + getCarsInScenraio();
        result = 31 * result + (getEvaluateCollisions() ? 1 : 0);
        result = 31 * result + (getEvaluateThroughput() ? 1 : 0);
        result = 31 * result + (isEvaluateDistanceTravelled() ? 1 : 0);
        result = 31 * result + (getConstrainCollisions() ? 1 : 0);
        result = 31 * result + (isRecordForPlayback() ? 1 : 0);
        result = 31 * result + (isEvaluateEk() ? 1 : 0);
        result = 31 * result + (isEvaluateStarvation() ? 1 : 0);
        result = 31 * result + (isEvaluateTinyThroughput() ? 1 : 0);
        temp = Double.doubleToLongBits(getSpeedLimit());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(getEvaluationSimulationTime());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(getAlgorithmTimeStepSize());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        result = 31 * result + (getAlgorithmName() != null ? getAlgorithmName().hashCode() : 0);
        result = 31 * result + (getSimulatorType() != null ? getSimulatorType().hashCode() : 0);
        result = 31 * result + (getTitle() != null ? getTitle().hashCode() : 0);
        result = 31 * result + (getExecutorSpecificProperties() != null ? getExecutorSpecificProperties().hashCode() : 0);
        result = 31 * result + (getObjectiveWeights() != null ? getObjectiveWeights().hashCode() : 0);
        result = 31 * result + (getGraphChosenIndividual() ? 1 : 0);
        result = 31 * result + (isImportantRun() ? 1 : 0);
        result = 31 * result + (isShowMETHistogram() ? 1 : 0);
        result = 31 * result + (isScenarioCreation() ? 1 : 0);
        result = 31 * result + (isHeadless() ? 1 : 0);
        result = 31 * result + (isStartScenario() ? 1 : 0);
        temp = Double.doubleToLongBits(getSpawnRateNorth());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(getSpawnRateEast());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(getSpawnRateWest());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        temp = Double.doubleToLongBits(getSpawnRateSouth());
        result = 31 * result + (int) (temp ^ (temp >>> 32));
        return result;
    }

    public boolean isRecordMoeaRunStatistics() {
        return true;
    }
}
