package jossihawk.simulator.sim;

import aim4.config.Debug;
import aim4.config.SimConfig;
import aim4.driver.pilot.V2IPilot;
import aim4.gui.Viewer;
import aim4.im.v2i.reservation.ReservationGridManager;
import aim4.map.GridMap;
import aim4.map.GridMapUtil;
import aim4.map.SpawnPoint;
import aim4.map.destination.RandomDestinationSelector;
import aim4.sim.setup.BasicSimSetup;
import jossihawk.simulator.SimulationProperties;

public class MOEASimSetup extends BasicSimSetup{
    SimulationProperties properties;
    public MOEASimSetup(BasicSimSetup setup, SimulationProperties properties ){
        super(setup);
        this.properties = properties;
    }

    private static boolean doubleCompare(double d1, double d2){
        return Math.abs(d1 - d2) < 0.01;
    }

    private static double resolveSpawnPointDirectionRate(SpawnPoint sp, SimulationProperties properties) {
        if (doubleCompare(sp.getHeading(), 0))
            return properties.getSpawnRateWest(); // fixed

        if (doubleCompare(sp.getHeading(), Math.PI / 2))
            return properties.getSpawnRateNorth(); //fixed

        if (doubleCompare(sp.getHeading(), Math.PI))
            return properties.getSpawnRateEast();

        if (doubleCompare(sp.getHeading(), Math.PI * (3d / 2d)))
            return properties.getSpawnRateSouth(); //fixed

        throw new RuntimeException("No matchings. God damn radians");
    }

    public static void setSpawnRates(GridMap layout, SimulationProperties properties){
        for (SpawnPoint sp : layout.getSpawnPoints()) {
            sp.setVehicleSpecChooser(
                    new GridMapUtil.UniformSpawnSpecGenerator(resolveSpawnPointDirectionRate(sp, properties),
                            new RandomDestinationSelector(layout)));
        }
    }


    /**
     * {@inheritDoc}
     */
    @Override
    public MOEASimulator getSimulator() {
        double currentTime = 0.0;
        GridMap layout = new GridMap(currentTime,
                numOfColumns,
                numOfRows,
                laneWidth,
                speedLimit,
                lanesPerRoad,
                medianSize,
                distanceBetween);
        ReservationGridManager.Config gridConfig =
                new ReservationGridManager.Config(SimConfig.TIME_STEP,
                        SimConfig.GRID_TIME_STEP,
                        0.25,  // staticBufferSize
                        0.0,   // internalTileTimeBufferSize
                        0.25,   // edgeTileTimeBufferSize
                        true,  // edgeTileTimeBufferSize
                        1.0);  // granularity

        Debug.SHOW_VEHICLE_COLOR_BY_MSG_STATE = true;

        //GridMapUtil.setFCFSManagers(layout, currentTime, gridConfig);
        //GridMapUtil.setUniformRandomSpawnPoints(layout, trafficLevel);
        //GridMapUtil.setDirectionalSpawnPoints(layout, trafficLevel, trafficLevel / 10);
        setSpawnRates(layout, properties);

        V2IPilot.DEFAULT_STOP_DISTANCE_BEFORE_INTERSECTION =
                stopDistBeforeIntersection;
        return new MOEASimulator(layout);
    }

    public static void main(String[] args){
        BasicSimSetup simSetup
                = new BasicSimSetup(1, // columns
                1, // rows
                6, // lane width
                25.0, // speed limit
                3, // lanes per road
                1.5, // median size
                150, // distance between
                0.80, // traffic level //This parameter is ignored later on. We override it with direction spewcific spawn rates
                1.0 // stop distance before intersection
        );
        MOEASimSetup setup = new MOEASimSetup(simSetup,new SimulationProperties("test.properties"));
        new Viewer(setup, true);
    }

}
