package jossihawk.simulator.sim;

import aim4.vehicle.VehicleSimView;

import java.util.HashMap;
import java.util.Map;

public class SimUtils {
    public static Map<Integer, Integer> uTurnCheck = new HashMap<>(4);
    public static Map<Integer, Integer> leftTurns = new HashMap<>(4);
    public static Map<Integer, Integer> rightTurns = new HashMap<>(4);


    static {
        uTurnCheck.put(0,2);
        uTurnCheck.put(2,0);
        uTurnCheck.put(6,4);
        uTurnCheck.put(4,6);

        leftTurns.put(4,0);
        leftTurns.put(0,6);
        leftTurns.put(6,2);
        leftTurns.put(2,4);

        rightTurns.put(5,3);
        rightTurns.put(1,5);
        rightTurns.put(7,1);
        rightTurns.put(3,7);

    }

    public enum Direction{
        RIGHT, LEFT, STRAIGHT
    }

    private static int getIndexLaneIDFromId(int id){
        if(id % 2 == 0)
            return id;
        return id - 1;
    }

    public static boolean checkIsUturn(int sourceId, int destId){
        sourceId = getIndexLaneIDFromId(sourceId);
        destId = getIndexLaneIDFromId(destId);
        return uTurnCheck.get(sourceId) == destId;

    }

    public static Direction resolveDirection(int startingLane, int laneOfDestination){
        if(startingLane % 2 != 0)
            startingLane -= 1;
        laneOfDestination = getIndexLaneIDFromId(laneOfDestination);
        if(startingLane == laneOfDestination)
            return Direction.STRAIGHT;

        if(startingLane == 0) {
            if (laneOfDestination == 4)
                return Direction.RIGHT;
            return Direction.LEFT;
        }

        if(startingLane == 6) {
            if (laneOfDestination == 0)
                return Direction.RIGHT;
            return Direction.LEFT;
        }

        if(startingLane == 2) {
            if (laneOfDestination == 6)
                return Direction.RIGHT;
            return Direction.LEFT;
        }

        if(startingLane == 4) {
            if (laneOfDestination == 2)
                return Direction.RIGHT;
            return Direction.LEFT;
        }

        throw new RuntimeException("Some mismatch in starting index lanes. WTF!?");

    }

    public static int resolveCorrectLaneIDfromCurrentLaneIDAndDirection(int laneID, Direction dir){
        if (dir == Direction.STRAIGHT)
            return laneID;
        if (laneID == getIndexLaneIDFromId(laneID)) //We are in a left bound lane
            return leftTurns.get(laneID);
        return rightTurns.get(laneID);

        /*switch (dir){
            case LEFT:
                if(laneID % 2 != 0)
                    laneID -= 1;
                return laneID;
            case RIGHT:
                if(laneID % 2 == 0)
                    laneID += 1;
                return laneID;
            default:
                return laneID;
        }*/
    }

    public static void main(String[] args) {
        System.out.println(resolveCorrectLaneIDfromCurrentLaneIDAndDirection(5,Direction.STRAIGHT));
        System.out.println(resolveDirection(5,3));
    }

    public static double distance(VehicleSimView v1, VehicleSimView v2){
        double xDist = v1.getPosition().getX() - v2.getPosition().getX();
        double yDist = v1.getPosition().getY() - v2.getPosition().getY();

        return Math.sqrt(Math.pow(xDist, 2) + Math.pow(yDist,2));

    }


}
