package jossihawk.simulator.sim.playback;

import jossihawk.simulator.sim.MOEASimulationState;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;

@XmlRootElement
public class SimulationPlayback {

    @XmlElement
    private final List<MOEASimulationState> states;

    public SimulationPlayback(){
        states = new ArrayList<MOEASimulationState>();
    }

    /**
     * Reads a stored simulation run from a given xml file.
     * @param file
     * @throws FileNotFoundException
     */
    public static SimulationPlayback readFromFile(File file) throws FileNotFoundException, JAXBException {
        JAXBContext context = JAXBContext.newInstance(SimulationPlayback.class);
        Unmarshaller m = context.createUnmarshaller();
        return (SimulationPlayback) m.unmarshal(file);
    }

    public void addState(MOEASimulationState state){
        states.add(state);
    }

    /**
     * Takes a File object, then writes its current state as xml to said file.
     * Useful for storing playbacks for later use.
     */
    public void writeToFile(File file) throws JAXBException {
        JAXBContext context = JAXBContext.newInstance(SimulationPlayback.class);
        Marshaller m = context.createMarshaller();
        m.marshal(this, file);
    }

    public List<MOEASimulationState> getStates(){return states;}


    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof SimulationPlayback)) return false;

        SimulationPlayback that = (SimulationPlayback) o;

        return !(states != null ? !states.equals(that.states) : that.states != null);

    }

    @Override
    public int hashCode() {
        return states != null ? states.hashCode() : 0;
    }
}
