package jossihawk.simulator.sim.stats;

import jossihawk.simulator.sim.MOEASimulator;
import jossihawk.simulator.sim.VehicleState;

/**
 * Listens for internal changes in the simulator related to logging of performance
 */
public abstract class SmallScaleStatisticsEventListener {

    private final MOEASimulator simulator;


    public SmallScaleStatisticsEventListener(MOEASimulator simulator) {
        simulator.registerSmallStepStatisticsListener(this);
        this.simulator = simulator;
    }

    public MOEASimulator getSimulator() {
        return simulator;
    }

    /**
     * Signals that a vehicle has been spawned
     * @param timeStep the spawn time
     * @param vin the vehicle VIN
     */
    abstract public void vehicleSpawned(double timeStep, int vin);

    abstract public void vehicleRemoved(double timeStep, int vin);

    abstract public void vehicleMoved(double timeStep, VehicleState stateBefore, VehicleState stateAfter);

    abstract public void vehiclesCollide(double timeStep, VehicleState vehicle1, VehicleState vehicle2);


}
