package jossihawk.simulator.sim_view;

import aim4.config.SimConfig;
import aim4.sim.setup.BasicSimSetup;
import jossihawk.simulator.SimulationProperties;
import jossihawk.simulator.sim.MOEASimSetup;
import jossihawk.simulator.sim.MOEASimulator;

import javax.swing.*;
import java.awt.*;

/**
 * A class to see the events of the simulator. Based on aim4.gui.Viewer, but based on a callback MVC-like pattern
 */
public class Display  extends JFrame{
    private static String TITLEBAR_STRING = "MOEA AIM";
    private BasicSimSetup initSimSetup;
    SlimmerCanvas canvas;
    private double targetFrameRate;
    private final double DEFAULT_TARGET_FRAME_RATE = 25;

    private MOEASimulator simulator;

    private long nextFrameTime;
    public Display(final BasicSimSetup initSimSetup, MOEASimulator simulator) {
        super(TITLEBAR_STRING);
        this.initSimSetup = initSimSetup;
        this.simulator = simulator;

        // the frame rate cannot be not larger than the simulation cycle
        targetFrameRate =
                Math.min(DEFAULT_TARGET_FRAME_RATE, SimConfig.CYCLES_PER_SECOND);
        this.nextFrameTime = 0; // undefined yet.

        // Lastly, schedule a job for the event-dispatching thread:
        // creating and showing this application's GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {

            /**
             * {@inheritDoc}
             */
            public void run() {
                createAndShowGUI(initSimSetup);
            }

        });
    }

    private void createAndShowGUI(BasicSimSetup initSimSetup) {
        // Apple specific property.
        System.setProperty("apple.laf.useScreenMenuBar", "true");
        System.setProperty("com.apple.mrj.application.apple.menu.about.name",
                "AIM Viewer");
        // Make sure that the program quits when we close the window
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        // Build the GUI
        //createMenuBar();



        this.setLayout(new BorderLayout());
        this.setMinimumSize(new Dimension(1000,1000));
        createComponents();
        canvas.setMinimumSize(new Dimension(200,200));
        canvas.setVisible(true);
        this.add(canvas, BorderLayout.CENTER);


        pack(); // pick the layout and show it
        setVisible(true);
        //initGUIsetting();

        canvas.initWithGivenMap(initSimSetup.getSimulator().getMap());
        canvas.repaint();
    }

    private void createComponents() {
        canvas = new SlimmerCanvas(this);
    }

    public MOEASimulator getSimulator() {
        return simulator;
    }

    public static void main(String[] args){
        BasicSimSetup simSetup
                = new BasicSimSetup(1, // columns
                1, // rows
                6, // lane width
                1, // speed limit
                2, // lanes per road
                1.5, // median size
                150, // distance between
                0.30, // traffic level
                1.0 // stop distance before intersection
        );
        MOEASimSetup setup = new MOEASimSetup(simSetup, new SimulationProperties("test.properties"));
        Display display = new Display(setup, setup.getSimulator());
    }
}

