package jossihawk.utils;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.RefineryUtilities;

import java.awt.*;
import java.util.List;

public class ChartDrawer extends ApplicationFrame {

    public ChartDrawer(List<XYSeries> xySeriesList, String name) {
        super(name);
        XYDataset dataset = createDataset(xySeriesList);
        JFreeChart chart = createChart(dataset, name);
        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new Dimension(1000, 500));
        add(chartPanel);
        pack();
        RefineryUtilities.positionFrameRandomly(this);
        setVisible(true);
    }

    /**
     * Creates a sample dataset
     */
    private XYDataset createDataset(List<XYSeries> xySeriesList) {
        XYSeriesCollection dataset = new XYSeriesCollection();
        for (XYSeries xySeries : xySeriesList) {
            dataset.addSeries(xySeries);
        }
        return dataset;
    }

    /**
     * Creates a chart
     */
    private JFreeChart createChart(XYDataset dataset, String name) {
        JFreeChart chart = ChartFactory.createXYLineChart(name, "Time-step", "Objective(s)", dataset, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.lightGray);
        plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        return chart;
    }
}
