package jossihawk.utils;

import jossihawk.ep.ChosenIndividual;
import jossihawk.simulator.SimulationProperties;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.data.xy.XYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RectangleInsets;
import org.jfree.ui.RefineryUtilities;

import java.awt.*;
import java.util.ArrayList;
import java.util.List;

public class ChosenIndividualChartDrawer extends ApplicationFrame {

    private SimulationProperties simulationProperties;
    private List<XYSeries> series;
    private XYSeries chosenIndividualFitness;

    public ChosenIndividualChartDrawer(SimulationProperties simulationProperties) {
        super("ChosenIndividualChartDrawer");
        this.simulationProperties = simulationProperties;
        XYDataset dataset = createDataset();
        JFreeChart chart = createChart(dataset);
        ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new Dimension(1000, 500));
        add(chartPanel);
        pack();
        RefineryUtilities.centerFrameOnScreen(this);
        setVisible(true);

        chosenIndividualFitness.add(0, 0);
        for (XYSeries sery : series) {
            sery.add(0, 0);
        }
    }

    /**
     * Creates a sample dataset
     */
    private XYDataset createDataset() {
        chosenIndividualFitness = new XYSeries("ChosenIndividualFitness");
        XYSeriesCollection dataset = new XYSeriesCollection();
        dataset.addSeries(chosenIndividualFitness);
        List<String> objectiveNames = simulationProperties.getObjectiveNames();
        series = new ArrayList<XYSeries>();
        for (int i = 0; i < simulationProperties.getNumberOfObjectives(); i++) {
            XYSeries tempSeries = new XYSeries(objectiveNames.get(i));
            series.add(tempSeries);
            dataset.addSeries(tempSeries);
        }
        return dataset;
    }

    public void addChosenIndividual(int step, ChosenIndividual chosenIndividual) {
        if (chosenIndividual.getChosenSolution() != null) {
            chosenIndividualFitness.add(step, chosenIndividual.getSingleObjectiveValue());
            for (int i = 0; i < series.size(); i++) {
                series.get(i).add(step, chosenIndividual.getChosenSolution().getObjective(i));
            }
        }
    }

    /**
     * Creates a chart
     */
    private JFreeChart createChart(XYDataset dataset) {
        JFreeChart chart = ChartFactory.createXYLineChart("Chosen fitness", "Step", "Fitness", dataset, PlotOrientation.VERTICAL, true, true, false);
        XYPlot plot = (XYPlot) chart.getPlot();
        plot.setBackgroundPaint(Color.lightGray);
        plot.setAxisOffset(new RectangleInsets(5.0, 5.0, 5.0, 5.0));
        plot.setDomainGridlinePaint(Color.white);
        plot.setRangeGridlinePaint(Color.white);
        return chart;
    }
}
