package jossihawk.utils;

import jossihawk.ep.AIMNondominatedPopulation;
import jossihawk.ep.AIMSolution;
import jossihawk.ep.MOEARunStatistics;
import jossihawk.simulator.AIMStats;

import javax.xml.bind.JAXBException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

public class ExperimentFiveAnalyzer {

    private final File experimentDir;
    private final File analytics;
    private Map<String, AIMStats> statisticsMap;

    public ExperimentFiveAnalyzer(File experimentDir) throws FileNotFoundException, JAXBException {
        this.experimentDir = experimentDir;
        File[] experiments = experimentDir.listFiles(pathname ->
            new File(new File(pathname, "results"), "globalStats.xml").exists());
        statisticsMap = new HashMap<>();
        for (File experiment : experiments) {
            AIMStats stats = AIMStats.readFromFile(new File(new File(experiment, "results"),"globalStats.xml"));
            statisticsMap.put(experiment.getName(), stats);
        }
        analytics = new File(experimentDir, "analytics");
        if(!analytics.exists())
            analytics.mkdir();
    }

    public void writeStatistics() throws FileNotFoundException, UnsupportedEncodingException {
        for (String statsKey : statisticsMap.keySet()) {
            AIMStats stats = statisticsMap.get(statsKey);
            int moeaRunCount = 0;
            File statsDir = new File(analytics, statsKey);
            statsDir.mkdir();
            for (MOEARunStatistics moeaRunStatistics : stats.getMoeaRunStatistics()) {
                File moeaRunDir = new File(statsDir, Integer.toString(moeaRunCount));
                moeaRunDir.mkdir();
                File allIndividualsFile = new File(moeaRunDir, statsKey + "-" + moeaRunCount +"-allIndividuals.csv");
                PrintWriter allWriter = new PrintWriter(allIndividualsFile, "UTF-8");
                allWriter.println("dt,ek,starve,tt,collisions");
                for (Integer key : moeaRunStatistics.keySet()) {
                    File generationFile = new File(moeaRunDir,statsKey + "-" + moeaRunCount +"-"+ key.toString()+".csv");
                    PrintWriter writer = new PrintWriter(generationFile, "UTF-8");
                    writer.println("dt,ek,starve,tt,collisions");
                    AIMNondominatedPopulation nondominatedPopulation = moeaRunStatistics.get(key);
                    if(nondominatedPopulation.getAimSolutions() != null) {
                    for (AIMSolution aimSolution : nondominatedPopulation.getAimSolutions()) {
                            double[] objs = aimSolution.getObjectives();
                            double[] constrs = aimSolution.getConstraints();
                            writer.println(objs[0]+","+objs[1]+","+objs[2]+","+objs[3]+","+constrs[0]);
                            allWriter.println(objs[0]+","+objs[1]+","+objs[2]+","+objs[3]+","+constrs[0]);
                        }
                    }
                    writer.close();
                }
                allWriter.close();
                moeaRunCount++;
            }
        }
    }

    public static void main(String[] args) throws FileNotFoundException, JAXBException, UnsupportedEncodingException {
        File experimentsDir = new File("experiments");
        ExperimentFiveAnalyzer analyzer = new ExperimentFiveAnalyzer(new File(experimentsDir, "ex5fixed"));
        analyzer.writeStatistics();
    }
}
