package jossihawk.utils;

import org.moeaframework.util.io.FileUtils;

import java.io.File;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.Properties;
import java.util.Scanner;

public class ExperimentOneGenerator {

    public static void generateScenarios(File sourceSetupDirectory, File destinationSetupDirectory) {
        if (destinationSetupDirectory.exists() && destinationSetupDirectory.list().length == 0) {
            File[] propertiesList = sourceSetupDirectory.listFiles(new FilenameFilter() {
                @Override
                public boolean accept(File dir, String name) {
                    return name.endsWith(".properties");
                }
            });
            File[] scenarioDirectories = sourceSetupDirectory.listFiles(new FilenameFilter() {
                @Override
                public boolean accept(File dir, String name) {
                    return new File(dir, name).isDirectory();
                }
            });
            int scenarioNumber = 0;
            for (int i = 0; i < propertiesList.length; i++) {
                File properties = propertiesList[i];
                for (int j = 0; j < scenarioDirectories.length; j++) {
                    File stateDirectory = scenarioDirectories[j];
                    // stateDirectory is a directory containing all folders for either high-, medium- or low-traffic
                    //stateDir is the directory containing each specific state and properties used for generating the state
                    for (File stateDir : stateDirectory.listFiles()) {
                        if (stateDir.listFiles().length == 2) {
                            File state = stateDir.listFiles(new FilenameFilter() {
                                @Override
                                public boolean accept(File dir, String name) {
                                    return name.endsWith(".xml");
                                }
                            })[0];
                            File generatedProperties = stateDir.listFiles(new FilenameFilter() {
                                @Override
                                public boolean accept(File dir, String name) {
                                    return name.endsWith(".properties");
                                }
                            })[0];
                            String scenarioName = scenarioNumber + "-" + stateDirectory.getName() + "-" + stateDir.getName() + "-" + "timestep-" + properties.getName().replace(".properties", "");
                            File scenarioDir = new File(destinationSetupDirectory, scenarioName);
                            scenarioDir.mkdir();
                            try {
                                FileUtils.copy(properties, new File(scenarioDir, scenarioDir.getName() + ".properties"));
                                FileUtils.copy(state, new File(scenarioDir, "state.xml"));
                                FileUtils.copy(generatedProperties, new File(scenarioDir, "usedForGenerating.properties"));
                                Properties scenarioProperties = new Properties();
                                scenarioProperties.setProperty("trafficDensity", stateDirectory.getName());
                                scenarioProperties.setProperty("state", stateDir.getName());
                                File propertiesFile = new File(scenarioDir, "additional.properties");
                                FileOutputStream fileOutputStream = new FileOutputStream(propertiesFile);
                                scenarioProperties.store(fileOutputStream, "Additional properties");
                                fileOutputStream.close();
                            } catch (IOException e) {
                                e.printStackTrace();
                            }
                            scenarioNumber++;
                        }
                    }
                }
            }
        } else System.out.println("Aborted! Directory is either not created or non-empty.");
    }

    public static void removeResults(File experimentDirectory) {
        Scanner scanner = new Scanner(System.in);
        System.out.println("Are you sure you want to delete all results in " + experimentDirectory.getAbsolutePath() + "?");
        System.out.println("0 for no\n1 for yes");
        System.out.print("$ ");
        int ans = scanner.nextInt();
        if (ans == 1) {
            File[] scenarios = experimentDirectory.listFiles();
            int resultCount = 0;
            int fileCount = 0;
            for (int i = 0; i < scenarios.length; i++) {
                File scenario = scenarios[i];
                File results = new File(scenario, "results");
                if (results.exists()) {
                    File[] files = results.listFiles();
                    for (int j = 0; j < files.length; j++) {
                        File file = files[j];
                        file.delete();
                        fileCount++;
                    }
                    results.delete();
                    resultCount++;
                }
            }
            System.out.println("Deleted " + resultCount + " results.");
            System.out.println("Deleted " + fileCount + " files.");
        } else System.out.println("Puh!");
    }

    public static void main(String[] args) {
        ExperimentOneGenerator.generateScenarios(new File("setups" + File.separator + "ex1v8_5runs"), new File("experiments" + File.separator + "ex1p1_5runs"));
//        removeResults(new File("experiments" + File.separator + "ex1"));
    }

}
