package jossihawk.utils;

import jossihawk.simulator.AIMStats;

import javax.xml.bind.JAXBException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

public class ExperimentTwoAnalyzer {

    private List<AIMStats> results;
    private List<Properties> properties;

    public ExperimentTwoAnalyzer(File experimentDir) throws IOException, JAXBException {
        results = new ArrayList<>();
        properties = new ArrayList<>();
        File[] resultFiles = experimentDir.listFiles(pathname -> {
            return new File(pathname + File.separator + "results" + File.separator + "globalStats.xml").exists();
        });
        System.out.println(resultFiles.length);

        for (int i = 0; i < resultFiles.length; i++) {
            File resultfile = new File(resultFiles[i].getAbsolutePath() + File.separator + "results" + File.separator + "globalStats.xml");
            results.add(AIMStats.readFromFile(resultfile));
            properties.add(readPropertiesFile(new File(resultFiles[i].getAbsolutePath() + File.separator + resultFiles[i].getName() + ".properties")));
        }
    }

    private Properties readPropertiesFile(File propertiesFile) throws IOException {
        FileInputStream fileInput = new FileInputStream(propertiesFile);
        Properties properties = new Properties();
        properties.load(fileInput);
        fileInput.close();
        return properties;
    }

    public void printResults() {
        for (int i = 0; i < results.size(); i++) {
            AIMStats result = results.get(i);
            Properties prop = properties.get(i);
            printResult(Double.parseDouble(prop.get("spawnRateNorth").toString()), result.getMeanEvacTime(), result.getNofChrashedVehicles(), result.getThroughput(), result.getTotalSpentEk());
        }
    }

    private void printResult(double spawnRate, double meanEvacTime, int nofChrashedVehicles, int throughput, double totalSpentEk) {
        System.out.format("Spawn: %10.6f   ", spawnRate);
        System.out.format("MET: %8.5f   ", meanEvacTime);
        System.out.format("Crash: %3d   ", nofChrashedVehicles);
        System.out.format("Throughput: %5d   ", throughput);
        System.out.format("Total Ek: %10.3f   ", totalSpentEk);
        System.out.println();
    }

    public static void main(String[] args) throws IOException, JAXBException {
        ExperimentTwoAnalyzer analyzer = new ExperimentTwoAnalyzer(new File("experiments" + File.separator + "ex2v10"));
        analyzer.printResults();
    }
}
