package jossihawk.utils;

import jossihawk.simulator.sim.stats.VehicleStatistics;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.xy.IntervalXYDataset;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.ApplicationFrame;
import org.jfree.ui.RefineryUtilities;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

public class HistogramChartDrawer extends ApplicationFrame {

    private XYSeries series;

    public HistogramChartDrawer(String title) {
        super(title);
        IntervalXYDataset dataset = createDataset();
        JFreeChart chart = createChart(dataset);
        final ChartPanel chartPanel = new ChartPanel(chart);
        chartPanel.setPreferredSize(new java.awt.Dimension(500, 270));
        setContentPane(chartPanel);
        pack();
        RefineryUtilities.centerFrameOnScreen(this);
        setVisible(true);
    }

    private IntervalXYDataset createDataset() {
        series = new XYSeries("Evacuation time");
        return new XYSeriesCollection(series);
    }

    public void addData(Collection<VehicleStatistics> vehicleStatisticsList) {
        Map<Integer, Integer> histogramData = new HashMap<Integer, Integer>();
        for (VehicleStatistics vehicleStatistics : vehicleStatisticsList) {
            if (vehicleStatistics.getEvacuationTime() != 0.0) {
                int evacuationTime = (int) Math.round(vehicleStatistics.getEvacuationTime() - vehicleStatistics.getSpawnTime());
                if (histogramData.containsKey(evacuationTime)) {
                    histogramData.put(evacuationTime, histogramData.get(evacuationTime) + 1);
                } else histogramData.put(evacuationTime, 1);
            }
        }
        series.clear();
        for (Map.Entry<Integer, Integer> seriesData : histogramData.entrySet()) {
            series.add(seriesData.getKey(), seriesData.getValue());
        }
    }

    private JFreeChart createChart(IntervalXYDataset dataset) {
        final JFreeChart chart = ChartFactory.createXYBarChart(
                "Evacuation Histogram",
                "Evacuation time",
                false,
                "Car count",
                dataset,
                PlotOrientation.VERTICAL,
                true,
                true,
                false
        );
        return chart;
    }


}
